<?php

namespace Drupal\node_by_email;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Email To node batch class.
 */
class EmailToNode {
  use StringTranslationTrait;
  use DependencySerializationTrait;

  /**
   * Node by email mid to node service.
   *
   * @var \Drupal\node_by_email\MidToNodeService
   */
  protected $midToNodeService;

  /**
   * Messenger Service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Constructs a new EmailToNode object.
   *
   * @param \Drupal\node_by_email\MidToNodeService $midToNodeService
   *   The mid to node service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MidToNodeService $midToNodeService, MessengerInterface $messenger) {
    $this->midToNodeService = $midToNodeService;
    $this->messenger = $messenger;
  }

/**
 * Create node batch method.
 *
 * @param int $mid
 *   Mail ID to process.
 * @param int $total
 *   Total number of mails to process.
 * @param mixed $context
 *   Batch context.
 */
public function createNode($mid, $total, &$context) {
  // Initialize 'current' if it's not set in the batch context.
  if (!isset($context['sandbox']['current'])) {
      $context['sandbox']['current'] = 0;
      $context['sandbox']['max'] = $total;
      $context['sandbox']['success'] = [];
      $context['results'] = 0; // Initialize results count.
  }

  try {
      // Check if this $mid has already been processed successfully.
      if (in_array($mid, $context['sandbox']['success'])) {
        $context['sandbox']['current']++;
        // Update the results count.
        // $context['results']++;
        return;
      }

      // Process the node creation from $mid.
      $this->midToNodeService->createNodeFromMid($mid);

      // Update the results count.
      // $context['results']++;

      // Update batch context with progress information.
      $context['message'] = $this->t('Processed @current out of @total.', [
          '@current' => $context['sandbox']['current'] + 1,
          '@total' => $total,
      ]);

      // Update 'current' in sandbox.
      $current = $context['sandbox']['current']++;

      // Record successful processing.
      $context['sandbox']['success'][] = $mid;
      $context['results'] = $total;

      // Calculate the percentage of completion.
      if ($total > 0) {
          $context['finished'] = ($context['sandbox']['current'] / $total);
      }
  } catch (\Exception $e) {
      // Handle any exceptions that occur during node creation.
      $context['message'] = $this->t('Error processing mail ID @mid: @error', [
          '@mid' => $mid,
          '@error' => $e->getMessage(),
      ]);
      $context['finished'] = 1;
      $context['results'] = 0;
  }
}


/**
 * Finish callback to batch.
 *
 * @param bool $success
 *   Boolean value.
 * @param mixed $results
 *   Results sets.
 * @param mixed $operations
 *   Operation sets.
 */
public function createNodeFinishedCallback($success, $results, $operations) {
  if ($success) {
      // Use the count of 'results' to determine the number of nodes processed.
      $count = $results; // Use 'results' key from $results array.
      $this->messenger->addMessage($this->formatPlural(
          $count,
          'One node processed.',
          '@count nodes processed.'
      ));
  } else {
      $this->messenger->addError($this->t('Finished with an error.'));
  }
}


}
